
/* Chrysalide - Outil d'analyse de fichiers binaires
 * collection.c - équivalent Python du fichier "analysis/db/collection.c"
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "collection.h"


#include <pygobject.h>


#include <i18n.h>
#include <analysis/db/collection.h>



/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_db_collection_type(void)
{
    static PyMethodDef py_db_collection_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_db_collection_getseters[] = {

        { NULL }

    };

    static PyTypeObject py_db_collection_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.db.DbCollection",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide collection for DataBase collection",

        .tp_methods     = py_db_collection_methods,
        .tp_getset      = py_db_collection_getseters,

    };

    return &py_db_collection_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide....db.DbCollection'.   *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_db_collection(PyObject *module)
{
    PyTypeObject *py_db_collection_type;    /* Type Python 'DbCollection'  */
    int ret;                                /* Bilan d'un appel            */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_db_collection_type = get_python_db_collection_type();

    py_db_collection_type->tp_base = &PyGObject_Type;
    py_db_collection_type->tp_basicsize = py_db_collection_type->tp_base->tp_basicsize;

    if (PyType_Ready(py_db_collection_type) != 0)
        return false;

    Py_INCREF(py_db_collection_type);
    ret = PyModule_AddObject(module, "DbCollection", (PyObject *)py_db_collection_type);
    if (ret != 0) return false;

    dict = PyModule_GetDict(module);
    pygobject_register_class(dict, "DbCollection", G_TYPE_DB_COLLECTION, py_db_collection_type,
                             Py_BuildValue("(O)", py_db_collection_type->tp_base));

    return true;

}
